/*

 Copyright 2013, SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.wrapper;

import com.sun.jna.Memory;
import com.sun.jna.Pointer;
import java.util.HashSet;
import java.util.Set;

public interface Callbacks extends CallbacksInterface {

    public class Logger implements CallbackLogger {

        @Override
        public void record(Pointer fmiComponent, String logSource, int status, String category, String message, Pointer parameters) {
            System.out.print(logSource + " reports: " + category);
            System.out.format(" with status %d \n", status);
            if (parameters != null)
                System.out.println("There was a problem inside the fmu, but some information cannot be retrieved");
            else
                System.out.println(message);
        }
    }

    public class AllocateMemory implements CallbackAllocateMemory {

        private int MEMORY_OFFSET = 4;
        public static Set<Pointer> onMemoryAllocatedObjects = new HashSet<>();

        @Override
        public Pointer allocate(Size_T numberOfObjects, Size_T objectSize) {
            Memory memoryToAllocate = new Memory(fixNumerOfObjects(numberOfObjects.intValue()) * objectSize.intValue());
            memoryToAllocate.align(MEMORY_OFFSET);
            memoryToAllocate.clear();
            Pointer allocatedObject = memoryToAllocate.share(0);
            onMemoryAllocatedObjects.add(allocatedObject);
            return allocatedObject;
        }

        private int fixNumerOfObjects(int numberOfObjects) {
            return (numberOfObjects <= 0) ? 1 : numberOfObjects;
        }
    }

    public class FreeMemory implements CallbackFreeMemory {

        @Override
        public void free(Pointer object) {
            AllocateMemory.onMemoryAllocatedObjects.remove(object);
        }
    }

    public class StepFinished implements CallbackStepFinished {

        @Override
        public void apply(Pointer fmiComponent, int status) {
        }
    }
}
