/*

 Copyright 2013, SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi;

import org.javafmi.modeldescription.modelvariables.ScalarVariable;
import org.javafmi.modeldescription.typedefinitions.EnumerationType;
import org.javafmi.wrapper.FunctionalMockupInterface;

public class VariableReader {

    private FunctionalMockupInterface fmuProxy;

    public VariableReader(FunctionalMockupInterface fmuProxy) {
        this.fmuProxy = fmuProxy;
    }

    public Variable read(String variableName) {
        ScalarVariable modelVariable = fmuProxy.getModelDescription().getScalarVariable(variableName);
        if (isEnumeration(modelVariable))
            return buildEnumerationVariable(modelVariable);
        else
            return buildVariable(modelVariable);
    }

    private boolean isEnumeration(ScalarVariable modelVariable) {
        return modelVariable.getType() instanceof EnumerationType;
    }

    private Variable buildEnumerationVariable(ScalarVariable modelVariable) {
        return new Variable(modelVariable.getName(), fmuProxy.getEnumeration(modelVariable));
    }

    private Variable buildVariable(ScalarVariable modelVariable) {
        String name = modelVariable.getName();
        int valueReference = modelVariable.getValueReference();
        switch (modelVariable.getTypeName()) {
            case "Real":
                return new Variable<>(name, fmuProxy.getReal(valueReference));
            case "Integer":
                return new Variable<>(name, fmuProxy.getInteger(valueReference));
            case "Boolean":
                return new Variable<>(name, fmuProxy.getBoolean(valueReference));
            case "String":
                return new Variable<>(name, fmuProxy.getString(valueReference));
            default:
                break;
        }
        throw new RuntimeException("Unknown variable type: " + modelVariable.getTypeName());
    }
}
