/*

 Copyright 2013, SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi;

import org.javafmi.modeldescription.modelvariables.ScalarVariable;
import org.javafmi.wrapper.FunctionalMockupInterface;

public class VariableWriter {

    private FunctionalMockupInterface fmu;

    public VariableWriter(FunctionalMockupInterface fmu) {
        this.fmu = fmu;

    }

    public Status write(String name, Object value) {
        ScalarVariable modelVariable = fmu.getModelDescription().getScalarVariable(name);
        int valueReference = modelVariable.getValueReference();
        switch (modelVariable.getTypeName()) {
            case "Real":
                return fmu.setReal(valueReference, getDoubleValue(value));
            case "Integer":
                return fmu.setInteger(valueReference, (int) value);
            case "Boolean":
                return fmu.setBoolean(valueReference, (boolean) value);
            case "String":
                return fmu.setString(valueReference, (String) value);
            default:
                break;
        }
        return Status.ERROR;
    }

    private double getDoubleValue(Object value) {
        return Double.parseDouble(String.valueOf(value));
    }
}
